import cv2    # 导入OpenCV库，用于图像处理
import mediapipe as mp    # 导入MediaPipe库，用于机器学习模型的应用
import numpy as np    # 导入numpy库，用于数值计算

mp_drawing = mp.solutions.drawing_utils    # 从MediaPipe的解决方案中导入绘图工具
mp_selfie_segmentation = mp.solutions.selfie_segmentation    # 从MediaPipe的解决方案中导入自拍分割模型

# For webcam input:
BG_COLOR = (192, 192, 192) # 设置背景颜色为灰色
cap = cv2.VideoCapture(0)    # 打开摄像头，准备获取视频流

cap.set(cv2.CAP_PROP_FRAME_WIDTH, 320)    # 设置视频流的宽度为320
cap.set(cv2.CAP_PROP_FRAME_HEIGHT, 240)    # 设置视频流的高度为240
cap.set(cv2.CAP_PROP_BUFFERSIZE, 1)    # 设置视频流的缓冲区大小为1

# 设置窗口为全屏模式
cv2.namedWindow('MediaPipe Selfie Segmentation', cv2.WND_PROP_FULLSCREEN)
cv2.setWindowProperty('MediaPipe Selfie Segmentation', cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)

# 加载MediaPipe的自拍分割模型
with mp_selfie_segmentation.SelfieSegmentation(model_selection=1) as selfie_segmentation:
    bg_image = None
    while cap.isOpened():
        success, image = cap.read()
        if not success:
            print("Ignoring empty camera frame.")
            continue

        # 将图像翻转并转换为RGB格式
        image = cv2.cvtColor(cv2.flip(image, 1), cv2.COLOR_BGR2RGB)
        # 为提高性能，将图像标记为不可写
        image.flags.writeable = False
        # 使用自拍分割模型处理图像
        results = selfie_segmentation.process(image)

        # 将图像标记为可写，并将其转换回BGR格式
        image.flags.writeable = True
        image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)

        # 创建一个条件矩阵，用于区分图像中的前景（人物）和背景
        condition = np.stack((results.segmentation_mask,) * 3, axis=-1) > 0.1
        # 如果背景图像为空，则创建一个新的背景图像，并将其颜色设置为指定的背景颜色
        if bg_image is None:
            bg_image = np.zeros(image.shape, dtype=np.uint8)
            bg_image[:] = BG_COLOR
        # 根据条件矩阵生成输出图像，前景部分使用原图，背景部分使用指定的背景图像
        output_image = np.where(condition, image, bg_image)
        # 将输出图像逆时针旋转90度
        output_image = cv2.rotate(output_image, cv2.ROTATE_90_COUNTERCLOCKWISE)
        # 显示处理后的图像
        cv2.imshow('MediaPipe Selfie Segmentation', output_image)
        # 如果用户按下ESC键（ASCII值为27），则退出循环
        if cv2.waitKey(5) & 0xFF == 27:
            break

# 释放摄像头
cap.release()
